# the following libraries are necessary for this script
require(vcfR)
require(pcaMethods)
require(rrBLUP)
require(qqman)
require(glmnet)

# read compressed vcf file (*.vcf.gz)
vcf <- read.vcfR("HDRA-G6-4-RDP1-RDP2-NIAS.AGCT_MAF005MIS001NR.vcf.gz")

# extract genotype data from vcf
gt <- extract.gt(vcf)

# get marker information (chromosone numbers and positions)
chrom <- getCHROM(vcf)
pos <- getPOS(vcf)

# show the first 10 SNPs of the first 10 lines
gt[1:10, 1:10]

# create a matrix of gt scores
gt.score <- matrix(NA, nrow(gt), ncol(gt))
gt.score[gt == "0/0"] <- -1
gt.score[gt == "0/1"] <- 0
gt.score[gt == "1/1"] <- 1

# name the rows and columns of matrix
rownames(gt.score) <- rownames(gt)
colnames(gt.score) <- colnames(gt)

# transpose the matrix
gt.score <- t(gt.score)

# read phenotype and line information
line <- read.csv("RiceDiversityLine4GWASGS.csv", row.names = 1)
pheno <- read.csv("RiceDiversityPheno4GWASGS.csv", row.names = 1)

# show the first 6 rows of the datasets
head(line)
head(pheno)

# extract marker genotypes of lines with phenotypic data
gt.score.wp <- gt.score[rownames(pheno),]
dim(gt.score.wp)

# perform probablistic principal component analysis (ppca) with the pca function
# this method allows the existence of missing values in the data
pca <- pca(gt.score.wp, "ppca", nPcs = 6)
plot(scores(pca)[,1:2], col = line$Sub.population)
legend("bottomleft", levels(line$Sub.population), col = 1:nlevels(line$Sub.population), pch = 1)
plot(scores(pca)[,3:4], col = line$Sub.population)
plot(scores(pca)[,5:6], col = line$Sub.population)

# prepare genotype data for GWAS function
g <- data.frame(colnames(gt.score.wp), as.numeric(chrom), pos, t(gt.score.wp))
rownames(g) <- 1:nrow(g)
colnames(g) <- c("marker", "chrom", "pos", rownames(gt.score.wp))

# calculate the additive relational matrix of 388 lines
amat <- A.mat(gt.score.wp, shrink = T)
colnames(amat) <- rownames(amat) <- rownames(gt.score.wp)
amat[1:6, 1:6]

# select a trait (actually, you can select multiple traits here)
y <- pheno$Seed.length.width.ratio

# prepare phenotypic value for GWAS
p <- data.frame(rownames(gt.score.wp), y)
colnames(p) <- c("gid", "y")

# GWAS with Q (population structure) + K (kinship relatedness)
# minor allele frequencuy is set as 0.05
gwa <- GWAS(p, g, K = amat, n.PC = 4, min.MAF = 0.05, plot = F)

# show the result of GWAS for the first six SNPs
head(gwa)

# calculate maf by ourselves
p.al1 <- apply(gt.score.wp + 1, 2, mean, na.rm = T) / 2
maf <- pmin(p.al1, 1 - p.al1)
head(maf)

# check the relationship and replace -logp of 0 with NA
gwa$y[maf < 0.05][1:50]

# replace -log P = 0 with NA
gwa$y[gwa$y == 0] <- NA

# draw manhattan plot
mht <- data.frame(SNP = gwa$marker, CHR = gwa$chrom, BP = gwa$pos, P = 10^(-gwa$y))
mht <- na.omit(mht)
manhattan(mht)

# draw qq plot
qq(mht$P)

# calculate false discovery rate
fdr <- p.adjust(mht$P, method = "BH")

# select markers under the threshold
fdr.thresh = 0.05
sig <- mht[fdr < fdr.thresh, ]
dim(sig)
sig

# draw manhattan plot with coloring significant SNPs
manhattan(mht, highlight = sig$SNP)

# close up the 3rd, 5th, and 7th chromosomes
manhattan(subset(mht, CHR %in% c(3,5,7)), highlight = sig$SNP)

# calculate false discovery rate
p.bonferroni <- p.adjust(mht$P, method = "bonferroni")

# select markers under the threshold
p.thresh = 0.05
sig <- mht[p.bonferroni < p.thresh, ]
dim(sig)
sig

# draw manhattan plot with coloring significant SNPs
manhattan(mht, highlight = sig$SNP)

# select a trait
# Number of fertile florets per plant
y <- pheno$Panicle.number.per.plant * pheno$Florets.per.panicle * pheno$Panicle.fertility 

# prepare phenotypic value for GWAS
p <- data.frame(rownames(gt.score.wp), y)
colnames(p) <- c("gid", "y")

# GWAS with Q (population structure) + K (kinship relatedness)
# minor allele frequencuy is set as 0.05
gwa <- GWAS(p, g, K = amat, n.PC = 4, min.MAF = 0.05, plot = F)

# replace -log P = 0 with NA
gwa$y[gwa$y == 0] <- NA

# draw manhattan plot
mht <- data.frame(SNP = gwa$marker, CHR = gwa$chrom, BP = gwa$pos, P = 10^(-gwa$y))
mht <- na.omit(mht)
manhattan(mht)

# calculate false discovery rate
fdr <- p.adjust(mht$P, method = "BH")
fdr.thresh = 0.05
sig <- mht[fdr < fdr.thresh, ]
sig

########## From here, we are going into genomic selection (GS) #######################

# impute missing data in gt.score
gt.score.imp <- gt.score
for(i in 1:ncol(gt.score)) {
	gt.score.imp[,i][is.na(gt.score[,i])] <- mean(gt.score[,i], na.rm = T)
}

# separate gt.score.imp into datasets with and without phenotypic data
gt.score.imp.wp <- gt.score.imp[rownames(pheno),]
gt.score.imp.wop <- gt.score.imp[!(rownames(gt.score.imp) %in% rownames(pheno)), ]
dim(gt.score.imp.wop)

# perform PCA
pca <- pca(gt.score.imp.wp, "ppca", nPcs = 6)
pca.wop <- predict(pca, gt.score.imp.wop)
plot(scores(pca)[,1:2], col = line$Sub.population)
points(pca.wop$scores[,1:2], col = "gray")
legend("bottomright", levels(line$Sub.population), col = 1:nlevels(line$Sub.population), pch = 1)
plot(scores(pca)[,3:4], col = line$Sub.population)
points(pca.wop$scores[,3:4], col = "gray")
plot(scores(pca)[,5:6], col = line$Sub.population)
points(pca.wop$scores[,5:6], col = "gray")

# prepare y again
y <- pheno$Panicle.number.per.plant * pheno$Florets.per.panicle * pheno$Panicle.fertility # Fertile florets per plant

# develop a prediction model with ridge regression
alpha = 0  # 0 corresponds to ridge regression

# select only samples with non-missing phenotype data
selector <- !is.na(y)
y <- y[selector]
X <- gt.score.imp.wp[selector,]

# select only polymorphic markers
poly <- apply(X, 2, var) > 0
X <- X[, poly]

# extract also the chromosome id info of the selected markers
X.chrom <- chrom[poly]  

# select the same markers also for lines without phenotypic data
X.wop <- gt.score.imp.wop[, poly]

# build a model with ridge regression
model.ridge <- cv.glmnet(X, y, alpha = alpha, standardize = F)

# predict y based on X (but actually this is not prediction)
y.fit <- predict(model.ridge, newx = X, s = "lambda.min") 

# compare fitted y with observed y
plot(y, y.fit)
abline(0, 1)
cor(y, y.fit)

# prepare ids (1 to n.fold) for all lines and sort them
n.fold = 5
id <- 1:length(y) %% n.fold + 1
id
cv.id <- sample(id)
cv.id

#' closs validate
y.train <- y[cv.id != 1]
X.train <- X[cv.id != 1, ]

#' remove monomorphic markers
poly <- apply(X.train, 2, var) > 0
X.train <- X.train[, poly]

# build a model
model <- cv.glmnet(X.train, y.train, alpha = alpha, standardize = F)

# predict y of the 1st fold
y.pred <- predict(model, newx = X[cv.id == 1, poly], s = "lambda.min")

# compare prediction with observation
plot(y[cv.id == 1], y.pred)
abline(0, 1)
cor(y[cv.id == 1], y.pred)

# repeat this analysis for #1 - #5 folds
y.pred <- rep(NA, length(y))
for(i in 1:n.fold) {
	print(i)
	y.train <- y[id != i]
	X.train <- X[id != i, ]
	
	poly <- apply(X.train, 2, var) > 0
	X.train <- X.train[, poly]
	
	model <- cv.glmnet(X.train, y.train, alpha = alpha, standardize = F)
	y.pred[id == i] <- predict(model, newx = X[id == i, poly], s = "lambda.min")
}

# comparison between prediction and observation
plot(y, y.pred)
abline(0, 1)
cor(y, y.pred)

# estimate and draw marker effects
model.ridge <- cv.glmnet(X, y, alpha = alpha, standardize = F)  # use all data
beta <- coef(model.ridge, s = "lambda.min")[-1]
plot(abs(beta), col = X.chrom, main = "Ridge", ylab = "Coefficients", xlab = "Position (bp)")

# prediction of phenotypes of genotypes without phenotypic records
y.pred.wop <- predict(model.ridge, newx = X.wop, s = "lambda.min")
conf <- hist(c(y.pred,y.pred.wop), col = "#ff00ff40", border = "#ff00ff")
hist(y.pred.wop, col = "#0000ff40", border = "#0000ff", breaks = conf$breaks, add = T)
sum(y.pred.wop>16)

# buid a model with LASSO
alpha = 1  # do the same thing with alpha = 1

y.pred <- rep(NA, length(y)) 
for(i in 1:n.fold) {
	print(i)
	y.train <- y[id != i]
	X.train <- X[id != i, ]
	
	poly <- apply(X.train, 2, var) > 0
	X.train <- X.train[, poly]
		
	model <- cv.glmnet(X.train, y.train, alpha = alpha, standardize = F)
	y.pred[id == i] <- predict(model, newx = X[id == i, poly], s = "lambda.min")
}
plot(y, y.pred)
abline(0,1)
cor(y, y.pred) 

# estimate and draw marker effects
model.lasso <- cv.glmnet(X, y, alpha = alpha, standardize = F)
beta <- coef(model.lasso, s = "lambda.min")[-1]
plot(abs(beta), col =X.chrom, main = "LASSO", ylab = "Coefficients", xlab = "Position (bp)")

# prediction of phenotypes of lines without phenotypic data
y.pred.wop <- predict(model.lasso, newx = X.wop, s = "lambda.min")
conf <- hist(c(y.pred,y.pred.wop), col = "#ff00ff40", border = "#ff00ff")
hist(y.pred.wop, col = "#0000ff40", border = "#0000ff", breaks = conf$breaks, add = T)
sum(y.pred.wop>16)